-----------------------------------------------------------------------
-- Get local references to certain often-used global objects for better speed
-----------------------------------------------------------------------
local _G = getfenv(0)

local math = _G.math
local table = _G.table

local ipairs = _G.ipairs
local pairs = _G.pairs
local setmetatable = _G.setmetatable
local tonumber = _G.tonumber
local tostring = _G.tostring
local unpack = _G.unpack
local math_sin, math_cos, abs, max = math.sin, math.cos, math.abs, math.max
local twoPi = math.pi * 2;
local atan2 = math.atan2;

-----------------------------------------------------------------------
-- WoW passes in the addon name and a private table, so grab those
-----------------------------------------------------------------------
local ADDON_NAME, private = ...

local Jughead = LibStub("AceAddon-3.0"):NewAddon("Jughead", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0" ) -- , "AceHook-3.0", "AceBucket-3.0"
Jughead.version = _G.GetAddOnMetadata(ADDON_NAME, "Version")

local Astrolabe = _G.DongleStub("Astrolabe-1.0")

-- Allow other people to find me.. if for some reason they want to anyway
_G["Jughead"] = Jughead
local Archy = _G["Archy"]
local GatherMate = LibStub("AceAddon-3.0"):GetAddon("GatherMate2")

local L = LibStub("AceLocale-3.0"):GetLocale("Jughead", true)

Jughead.enabled = true
Jughead.lastDigInSiteSurveyed = false
Jughead.frameCreated = false

SITES_PER_CONTINENT = 4
SURVEY_GREEN = 1
SURVEY_YELLOW = 2
SURVEY_RED = 3


local PROFILE_DEFAULTS = {
	profile = {
		general = {
			enabled = true,
			visible = true,
			useArchy = true,
			useGM2 = true
		},
		frame = {
			attachToArchy = true,
			position = {
				"TOPLEFT",
				"TOPLEFT",
				10,
				40
			},
			anchor = "TOPLEFT",
			positionX = 10,
			positionY = 40,
			scale = 0.75,
			alpha = 1,
			style = "Compact"
		},
	},
}



-----------------------------------------------------------------------
-- "Prototype" some functions so we can use them
-----------------------------------------------------------------------
local dumpMinimapStats
local fetchArchySurveyPOI
local fetchGM2SurveyPOI
--local CacheMapData

-----------------------------------------------------------------------
-- Some variables for keeping track of things
-----------------------------------------------------------------------
local current_continent  -- Where the heck am I anyway?  Updated by timer
local timer_handle -- used to keep track of player location
local race_data = {}  -- used to keep track of how many fragments we have of each race, so we can know when to turn POI frames back on.
local myButtonFrame   -- holds my frame of red/yellow/green/reset buttons
local SURVEY_YARDS = {[SURVEY_GREEN] = 42, [SURVEY_YELLOW] = 82, [SURVEY_RED] = 800}  -- Need a little wiggle room with these.
local SURVEY_ANGLES = {[SURVEY_GREEN] = {left=math.rad(20),right=math.rad(340)}, [SURVEY_YELLOW] = {left=math.rad(30),right=math.rad(330)}, [SURVEY_RED] = {left=math.rad(50),right=math.rad(310)}}
local archDigFrame -- The Archy dig site frame
local nearestDigsiteIndexer  -- The DIG_SITES index for the site that the player is closest to
local nextNearestDigsiteIndexer  -- The DIG_SITES index for the site that the player is closest to
local player_map, player_level, player_x, player_y -- updated by every UpdatePlayerPosition
local ZONE_DATA = {}
local MAP_CONTINENTS = {}


----------------------------------------
----------------------------------------
local function DebugPrint(msg)
--	Jughead:Print(msg)
end
----------------------------------------
----------------------------------------
fetchArchySurveyPOI = function()
	
	local kids = { Minimap:GetChildren() }
	local archyKids = {}

	for _, cFrame in pairs(kids) do
		local fName = cFrame:GetName()
		
		if fName and fName:match("ArchyMinimap_SurveyPOI%d+") then
			if cFrame.type == "survey" then
				table.insert(archyKids,cFrame)
			end
		end

	end
	
	return archyKids

end




----------------------------------------
----------------------------------------
local function enableAllArchySurveyPOI()
	local kids = fetchArchySurveyPOI()

	for _, cFrame in pairs(kids) do
		-- If archy got called before us, then it will have removed the siteid field from the POI
		-- in the case that the active dig site we were in is no longer valid (we just surveyed the last one in this site)
		if cFrame.siteId then
			cFrame.icon:Show()
			cFrame:Show()
		end
	end
end



----------------------------------------
-- minDist  POI closer to player than minDist will be hidden
-- maxDist  POI farther from player than maxDist will be hidden
----------------------------------------
local function hideArchySurveyPOI(minDist, maxDist, minAngle, maxAngle)

	local kids = fetchArchySurveyPOI()

	-- DebugPrint( "minDist=" .. minDist .. " maxDist=" .. maxDist .. " minAngle=" .. minAngle .. " maxAngle=" .. maxAngle )	

	local playerFacing = _G.GetPlayerFacing()
	local rad_360 = math.rad(360)  -- use this to normalize resulting angle
	
	-- DebugPrint( "GetPlayerFacing=" .. playerFacing .. " rad_360=" .. rad_360 )	

	for _, cFrame in pairs(kids) do
		-- This gives the angle, starting from North, moving CClkwise from the player point (extended to North) to the given icon
		local angle = Astrolabe:GetDirectionToIcon( cFrame )
		if angle then
			-- To adjust for angle RELATIVE TO PLAYER, we need to SUBTRACT the player facing 
			angle = angle - playerFacing
			if angle < 0 then
				angle = angle + rad_360
			end
	
			-- DebugPrint( cFrame:GetName() .. " adjusted angle=" .. angle )	
	
			local showIt = false
			
			if angle <= minAngle or angle >= maxAngle then
				-- ok, it's in the right set of angles, now look for distance
				
				local dist = Astrolabe:GetDistanceToIcon( cFrame )
				
				-- DebugPrint( cFrame:GetName() .. " distance=" .. dist )	
				
				if dist >= minDist and dist <= maxDist then
					showIt = true
				end
			
			
			end
			
			-- Do the arrow texture as well?
			if showIt then
				cFrame.icon:Show()
				cFrame:Show()
			else
				cFrame.icon:Hide()
				cFrame:Hide()
			end				
		
			-- DebugPrint( cFrame:GetName() .. "  " .. angle )		
		end
	end


end


----------------------------------------
----------------------------------------
local function fetchArchyDigSiteFrame()
	
	local kids = { _G.UIParent:GetChildren() }
	local retval
	
	-- DebugPrint( "Looking for ArchyDigSiteFrame.  Found " .. #kids .. " child frames" )
	
	for _, cFrame in pairs(kids) do
		local fName = cFrame:GetName()
		
		if fName and fName == "ArchyDigSiteFrame" then
			-- DebugPrint( "Found ArchyDigSiteFrame" )
			retval = cFrame
			break
		end
	end
	
	return retval

end





----------------------------------------
-- This is used to fetch all the map information, then once we have it, the function is
-- erased (nil) and no longer called.
-- It is called in UpdatePlayerPosition until we get the map data.
----------------------------------------
--CacheMapData = function()
--[[
	if not next(MAP_CONTINENTS) then MAP_CONTINENTS = { _G.GetMapContinents() } end
	for continent_id, continent_name in pairs(MAP_CONTINENTS) do
		_G.SetMapZoom(continent_id)
		local map_id = _G.GetCurrentMapAreaID()
		local map_file_name = _G.GetMapInfo()

		--MAP_ID_TO_CONTINENT_ID[map_id] = continent_id
		ZONE_DATA[map_id] = {
			continent = continent_id,
			map = map_id,
			level = 0,
			--mapFile = map_file_name,
			id = 0,
			name = continent_name
		}
		--MAP_FILENAME_TO_MAP_ID[map_file_name] = map_id
		--MAP_ID_TO_ZONE_NAME[map_id] = continent_name

		for zone_id, _ in pairs{ _G.GetMapZones(continent_id) } do
			_G.SetMapZoom(continent_id, zone_id)
			local map_id = _G.GetCurrentMapAreaID()
			local level = _G.GetCurrentMapDungeonLevel()
			local map_file_name = _G.GetMapInfo()
			--MAP_FILENAME_TO_MAP_ID[map_file_name] = map_id
			--MAP_ID_TO_ZONE_ID[map_id] = zone_id
			--MAP_ID_TO_ZONE_NAME[map_id] = zone_name
			--ZONE_ID_TO_NAME[zone_id] = zone_name
			ZONE_DATA[map_id] = {
				continent = continent_id,
				map = map_id,
				level = level,
				--mapFile = map_file_name,
				id = zone_id
				--name = zone_name
			}
		end
	end
	if next(ZONE_DATA) then
		CacheMapData = nil
	end
--]]
--end


----------------------------------------
-- Archy.db.global.surveyNodes[site.id]
-- The Archy survey database is keyed by dig site ID.
-- The dig site ID is is stored in site.id where site is an element of continent_digsites[current_continent]
--   current_continent is obtained in UpdatePlayerPosition and is obtained from _G.GetCurrentMapContinent()
--   continent_digsites is built up in UpdateSite, and continent_digsites[continent_id] is the result of GetContinentSites
--    GetContinentSites queries the map interface for a bunch of information, and it references an internal tables DIG_SITES to find
--     that the site_id is the blobid
--    DIG_SITES is indexed by the localized digsite name obtained by _G.GetMapLandmarkInfo(index)
--    The UpdatePlayerPosition does most of this work to populate nearestDigsiteIndexer and nextNearestDigsiteIndexer.
--    We need both because all the information we can get from the map API is the center of the digsite, and some of the sites
--    are close together and large (Hinterlands for example) which could cause all sorts of confusion.
----------------------------------------
local function userIsOnArchyNode()

-- Figure out siteID
-- Figure out map, floor, x and y
	local exists = false

	if not self.enabled or not self.archyFound then
		return
	end

	-- If there isn't a nearest indexer, then, well..
	if not nearestDigsiteIndexer then
		DebugPrint("nearestDigsiteIndexer is nil")
		return exists
	end
	

	local map, level, x, y = Astrolabe:GetCurrentPlayerPosition()
	
	local siteId = nearestDigsiteIndexer.id
	
	if Archy.db.global.surveyNodes[siteId] then
		for _, node in pairs(Archy.db.global.surveyNodes[siteId]) do
			local distance = Astrolabe:ComputeDistance(map, level, x, y, node.m, node.f, node.x, node.y)

			if not distance or _G.IsInInstance() then
				distance = 0
			end

			if distance <= 10 then
				DebugPrint("found a match in nearestDigsiteIndexer!")
				exists = true
				break
			end
		end
	end
	
	-- If there isn't a next nearest digsite, then return whatever we found with the nearest digsite
	-- Likewise, if we found something in the nearest digsite, then there's no way we could find something in the next nearest
	if (not nextNearestDigsiteIndexer) or (exists) then
		DebugPrint("nextNearestDigsiteIndexer is nil or we already found one")
		return exists
	end

	
	siteId = nextNearestDigsiteIndexer.id

	if not Archy.db.global.surveyNodes[siteId] then
		--DebugPrint( "Couldn't find an Archy digsite for siteId " .. siteId )
		return exists
	end

	for _, node in pairs(Archy.db.global.surveyNodes[siteId]) do
		local distance = Astrolabe:ComputeDistance(map, level, x, y, node.m, node.f, node.x, node.y)

		if not distance or _G.IsInInstance() then
			distance = 0
		end

		if distance <= 10 then
			DebugPrint("found a match in nextNearestDigsiteIndexer!")
			exists = true
			break
		end
	end
	
	return exists

end



----------------------------------------
----------------------------------------
function buttonRed_OnLClick()
	hideArchySurveyPOI( SURVEY_YARDS[SURVEY_YELLOW], SURVEY_YARDS[SURVEY_RED], SURVEY_ANGLES[SURVEY_RED].left, SURVEY_ANGLES[SURVEY_RED].right )
	--hideGM2SurveyPOI( SURVEY_YARDS[SURVEY_YELLOW], SURVEY_YARDS[SURVEY_RED], SURVEY_ANGLES[SURVEY_RED].left, SURVEY_ANGLES[SURVEY_RED].right )
end


----------------------------------------
----------------------------------------
function buttonYellow_OnLClick()
	hideArchySurveyPOI( SURVEY_YARDS[SURVEY_GREEN], SURVEY_YARDS[SURVEY_YELLOW], SURVEY_ANGLES[SURVEY_YELLOW].left, SURVEY_ANGLES[SURVEY_YELLOW].right )
	--hideGM2SurveyPOI( SURVEY_YARDS[SURVEY_GREEN], SURVEY_YARDS[SURVEY_YELLOW], SURVEY_ANGLES[SURVEY_YELLOW].left, SURVEY_ANGLES[SURVEY_YELLOW].right )
end

----------------------------------------
----------------------------------------
function buttonGreen_OnLClick()
	hideArchySurveyPOI( 4, SURVEY_YARDS[SURVEY_GREEN], SURVEY_ANGLES[SURVEY_GREEN].left, SURVEY_ANGLES[SURVEY_GREEN].right )
	--hideGM2SurveyPOI( 4, SURVEY_YARDS[SURVEY_GREEN], SURVEY_ANGLES[SURVEY_GREEN].left, SURVEY_ANGLES[SURVEY_GREEN].right )
end

----------------------------------------
----------------------------------------
function buttonReset_OnLClick()

	enableAllArchySurveyPOI()
	--enableAllGM2SurveyPOI()
	
end


local DIG_LOCATION_TEXTURE = 177
----------------------------------------
-- Called by UpdatePlayerPosition whenever continent_digsites[current_continent] is empty or nil
-- This happens on initial load and when we receive the ARTIFACT_DIG_SITE_UPDATED event or when the player changes continents
-- UpdatePlayerPosition only calls this when the world map frame is closed
----------------------------------------
local function GetContinentSites(continent_id)
	
	--DebugPrint( "In GetContinentSites" )
	
	local new_sites = {}
	local showDig = _G.GetCVarBool("digSites")
	if not showDig then
		_G.SetCVar("digSites", "1")
		_G.WorldMapArchaeologyDigSites:Show()
		_G.WorldMapShowDigSites:SetChecked(1)
		_G.RefreshWorldMap()
		showDig = "0"
	end
	
	if not continent_id then
		--DebugPrint( "continent_id was nil" )
		return new_sites
	end
	
	_G.SetMapZoom(continent_id)
	
	for index = 1, _G.GetNumMapLandmarks() do
		local name, description, texture_index, activeDigsiteX, activeDigsiteY = _G.GetMapLandmarkInfo(index)

		if texture_index == DIG_LOCATION_TEXTURE then
			local staticSiteData = private.dig_sites[name]
			continentMapID = Astrolabe:GetMapID(continent_id, 0)

			if staticSiteData then 
				siteMapID = staticSiteData.map
			
				local worldX, worldY = Astrolabe:TranslateWorldMapPosition(continentMapID, 0, activeDigsiteX, activeDigsiteY, siteMapID, 0)

				local digsite = {
					continent = continentMapID,
					map = siteMapID,
					level = 0,
					x = worldX,
					y = worldY,
					name = name,
					id = staticSiteData.blob_id,  -- This is the indexer into the digsites location database
					distance = 999999
				}
				table.insert(new_sites, digsite)
			end
		end
	end
	if showDig == "0" then -- restore initial setting
		_G.SetCVar("digSites", showDig)
		_G.WorldMapArchaeologyDigSites:Hide()
		_G.WorldMapShowDigSites:SetChecked(nil)
		_G.RefreshWorldMap()
	end
	return new_sites
end


----------------------------------------
----------------------------------------
local function SortSitesByDistance(a, b)
	return (a.distance or 0) < (b.distance or 0)
end


----------------------------------------
-- Called by UpdatePlayerPosition to find the nearest two digsites on the current continent
----------------------------------------
function setNearestDigsites()

	--DebugPrint( "In setNearestDigsites" )

	if not continent_digsites[current_continent] or (#continent_digsites[current_continent] == 0) then
		nearestDigsiteIndexer = nil
		nextNearestDigsiteIndexer = nil
		
		return
	end

	for index = 1, SITES_PER_CONTINENT do
		local site = continent_digsites[current_continent][index]

		site.distance = Astrolabe:ComputeDistance(player_map, player_level, player_x, player_y, site.map, site.level, site.x, site.y)
	end


	-- Sort sites
	local sites = continent_digsites[current_continent]
	table.sort(sites, SortSitesByDistance)
	
	local nindex = nil
	nindex, nearestDigsiteIndexer = next(sites,nil)
	_, nextNearestDigsiteIndexer = next(sites,nindex)
end


-----------------------------------------------------------------------
--
-- Event handlers and timers
--
-----------------------------------------------------------------------

-----------------------------------------------------------------------
-- Mouse events
-----------------------------------------------------------------------

----------------------------------------
----------------------------------------
function Jughead_ButtonRed_OnMouseDown(self, button)
--	if button == "LeftButton" then
		buttonRed_OnLClick()
--	elseif button == "RightButton" then
--		buttonRed_OnRClick()
--	end
end

----------------------------------------
----------------------------------------
function Jughead_ButtonYellow_OnMouseDown(self, button)
	buttonYellow_OnLClick()
end

----------------------------------------
----------------------------------------
function Jughead_ButtonGreen_OnMouseDown(self, button)
	buttonGreen_OnLClick()
end

----------------------------------------
----------------------------------------
function Jughead_ButtonReset_OnMouseDown(self, button)
	buttonReset_OnLClick()
end


-----------------------------------------------------------------------
-- Timers
-----------------------------------------------------------------------

----------------------------------------
-- Called by the timer we set up in PLAYER_ENTERING_WORLD event
----------------------------------------
function Jughead:UpdatePlayerPosition(force)

	if _G.IsInInstance() or _G.UnitIsGhost("player") then
		return
	end

	-- Please note that GetCurrentMapContinent shows the currently displayed continent on the world map.
	-- This is NOT NECESSARILY where the player is, if the player has the world map open.
	-- Therefore, if the world map is open, don't update current_continent...
	if WorldMapFrame:IsVisible() then
		return
	end

	if _G.GetCurrentMapAreaID() == -1 then
		return
	end

	current_continent = _G.GetCurrentMapContinent()

	player_map, player_level, player_x, player_y = Astrolabe:GetCurrentPlayerPosition()

	-- These functions will change the position of the world map, so save where we are now
	local original_map_id = _G.GetCurrentMapAreaID()

	-- Keep trying until we get all the map data
--	if CacheMapData then
--		CacheMapData()
--	end

	-- Update the list of active digsites on this continent, if necessary
	if not continent_digsites then
		continent_digsites = {}
	end
	
	if not continent_digsites[current_continent] or (#continent_digsites[current_continent] == 0) then
		continent_digsites[current_continent] = GetContinentSites(current_continent)
	
		--UpdateSite(current_continent)
	end
	_G.SetMapByID(original_map_id)
	
	if not player_map or not player_level or (player_x == 0 and player_y == 0) then
		return
	end

	setNearestDigsites()
	
end


-----------------------------------------------------------------------
-- Game events
-----------------------------------------------------------------------

----------------------------------------
-- When we enter the world, start keeping track of where the player is
----------------------------------------
function Jughead:PLAYER_ENTERING_WORLD()
	-- Two timers are needed here: If we force a call to UpdatePlayerPosition() too soon, the site distances will not update properly and the notifications may vanish just as the player is able to see them.
	if not timer_handle and self.enabled then
		self:ScheduleTimer(function()
			timer_handle = self:ScheduleRepeatingTimer("UpdatePlayerPosition", 5)
		end, 2)
	end

end


----------------------------------------
--[[ Survey Functions ]] --
-- Copied from Archy to interface with the Archy survey database
----------------------------------------
local function AddSurveyNode(siteId, map, level, x, y)
	local newNode = {
		m = map,
		f = level,
		x = x,
		y = y
	}
	local exists = false

	if not Archy.db.global.surveyNodes then
		Archy.db.global.surveyNodes = {}
	end

	if not Archy.db.global.surveyNodes[siteId] then
		Archy.db.global.surveyNodes[siteId] = {}
	end

	for _, node in pairs(Archy.db.global.surveyNodes[siteId]) do
		local distance = Astrolabe:ComputeDistance(newNode.m, newNode.f, newNode.x, newNode.y, node.m, node.f, node.x, node.y)

		if not distance or _G.IsInInstance() then
			distance = 0
		end

		if distance <= 10 then
			exists = true
			break
		end
	end
	if not exists then
		table.insert(Archy.db.global.surveyNodes[siteId], newNode)
	end
end


----------------------------------------
----------------------------------------
function Jughead:CURRENCY_DISPLAY_UPDATE()

	--DebugPrint( "In CURRENCY_DISPLAY_UPDATE" )

	if not current_continent or _G.GetNumArchaeologyRaces() == 0 or not self.enabled then
		return
	end

	

	local goodDig = false
	for race_id = 1, _G.GetNumArchaeologyRaces() do
		local _, _, _, currencyAmount = _G.GetArchaeologyRaceInfo(race_id)
		if ( not race_data[race_id] ) then
			race_data[race_id] = {}
		end
		local diff = currencyAmount - (race_data[race_id].currency or 0)

		race_data[race_id].currency = currencyAmount

		if diff < 0 then
			-- we've "spent" fragments by solving an artifact.. nothing to do here, move along..
		elseif diff > 0 then
			-- we've gained fragments so we have successfully dug.  Turn the survey nodes back on

			goodDig = true
			

		end
	end

	-- If we found something, and Archy is enabled, then turn the archy pins back on
	if goodDig and self.enabled and self.archyFound and not self.lastDigInSiteSurveyed then
		enableAllArchySurveyPOI()
	end

	self.lastDigInSiteSurveyed = false

--[[
	if goodDig then
		DebugPrint( "goodDig is true"  )
	end
	if self.enabled then
		DebugPrint( "self.enabled is true" )
	end
	if self.archyFound then
		DebugPrint( "self.archyFound is true" )
	end
	if self.gm2Found then
		DebugPrint( "self.gm2Found is true" )
	end	
	if nearestDigsiteIndexer then
		DebugPrint( "nearestDigsiteIndexer is not nil" )
		
		if nearestDigsiteIndexer.id then
			DebugPrint( "nearestDigsiteIndexer.id " .. nearestDigsiteIndexer.id )
		end
		if nearestDigsiteIndexer.map then
			DebugPrint( "nearestDigsiteIndexer.map " .. nearestDigsiteIndexer.map )
		end
	end
--]]

	-- If we found something, and both GM2 and Archy are enabled, see if we can copy any nodes from the
	-- GM2 database to the Archy database.
	if goodDig and self.enabled and self.archyFound and self.gm2Found and nearestDigsiteIndexer then

		local x, y = GetPlayerMapPosition("player")
		local zone = GetCurrentMapAreaID()
		local level = GetCurrentMapDungeonLevel()

		local numFound = 0
		for coord, nodeID in GatherMate:FindNearbyNode(zone, x, y, level, "Archaeology", 200) do
			numFound = numFound + 1

			-- Decode GM2 x,y,level
			local x2, y2, level2 = floor(coord/1000000)/10000, floor(coord % 1000000 / 100)/10000, coord % 100

			DebugPrint( "(" .. x2 .. "," .. y2 .. "," .. level2 .. ")")

			AddSurveyNode(nearestDigsiteIndexer.id, nearestDigsiteIndexer.map, level2, x2, y2)

		end

		DebugPrint( "Found " .. numFound .. " GM2 nodes nearby.")
	end
	
end


----------------------------------------
----------------------------------------
function Jughead:ARTIFACT_DIG_SITE_UPDATED()
	if not current_continent then
		return
	end

	-- This will cause UpdatePlayerPosition to requery sites and recompute distances	
	continent_digsites[current_continent] = nil
	
end


----------------------------------------
----------------------------------------
function Jughead:ConfigUpdated()

end

----------------------------------------
----------------------------------------
function Jughead:OnProfileUpdate(event, database, ProfileKey)
	if self.frameCreated then
		self:ConfigUpdated()
	end
end


----------------------------------------
----------------------------------------
function Jughead:UNIT_SPELLCAST_SUCCEEDED(event, unit, spell, rank, line_id, spell_id)
	if unit ~= "player" or spell_id ~= SURVEY_SPELL_ID then
		return
	end

	-- Look for the condition where the player just surveyed and found something.  At that point, the digsites have
	-- changed, and we want to avoid leaving artifacts on the minimap from the old digsite
	local lastNearestDigsiteIndexer = nearestDigsiteIndexer
	UpdatePlayerPosition()

	-- ok, digsites have changed..	
	if lastNearestDigsiteIndexer ~= nearestDigsiteIndexer then
		self.lastDigInSiteSurveyed = true  -- leave a marker for CURRENCY_DISPLAY_UPDATE
		enableAllArchySurveyPOI() -- reenable all the POIs so that there won't be any strangeness when Archy refreshes the minimap in CURRENCY_DISPLAY_UPDATE
	end
	
	
end

-----------------------------------------------------------------------
--
-- Addon events
--
-----------------------------------------------------------------------

----------------------------------------
----------------------------------------
function Jughead:OnInitialize()

	self.db = LibStub("AceDB-3.0"):New("JugheadDB", PROFILE_DEFAULTS, 'Default')
	self.db.RegisterCallback(self, "OnNewProfile", "OnProfileUpdate")
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileUpdate")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileUpdate")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileUpdate")

end

----------------------------------------
-- Called when the addon is enabled
----------------------------------------
function Jughead:OnEnable()

	self.archyFound = false
	self.gm2Found = false
	self.enabled = false
	
	-- Belt and suspenders..
	if not self.db.profile then
		self.db.profile = PROFILE_DEFAULTS
	end
	if not self.db.profile.general then
		self.db.profile.general = PROFILE_DEFAULTS.general
	end
	
	
	-- Decide if I found my target addons, and if the user has asked me to interface with them
	if _G["Archy"] and self.db.profile.general.useArchy then
		self.archyFound = true
	end

	if GatherMate and self.db.profile.general.useGM2 then
		self.gm2Found = true
	end

	if  (self.archyFound or self.gm2Found) and self.db.profile.general.enabled then
		self.enabled = true
	end


	-- If I found Archy, and the user wants me to use Archy, then hook myself up
	if self.enabled and self.archyFound then

		--DebugPrint( ADDON_NAME .. " " .. Jughead.version .. " " .. Archy.version )

		local archyVers1, archyVers2, archyVers3 = strsplit(".", Archy.version)
		
		-- Yeah, this is ugly.  I'll fix it later.
		local goodArchy = true
		if archyVers1 ~= "1" or archyVers2 ~= "8" then
			goodArchy = false
		elseif tonumber(archyVers3) then
			if tonumber(archyVers3) < 30 then
				goodArchy = false				
			end
		elseif archyVers3 ~= "31a" then
			goodArchy = false
		end

		if goodArchy == false then
			Jughead:Print("I don't know this Archy.  Use at your own risk." )		
		end

		if Archy.db and Archy.db.global and Archy.db.global.surveyNodes then
			DebugPrint("I found the Archy survey database.")
		end
		
		-- Archy frames might not be created at this point yet, so delay a bit..		
		if self.db.profile.frame.attachToArchy then
			self:ScheduleTimer(function()
				-- Find the ArchyDistanceIndicator frame
				local archDigFrame = fetchArchyDigSiteFrame()

				-- Now, attach our frame to it		
				if archDigFrame then
					--DebugPrint( "Found Archy's dig frame.")
					myButtonFrame = _G.CreateFrame("Frame", "JugheadDistanceAttachmentFrame", archDigFrame, "JugheadDistanceAttachment")
					myButtonFrame:Show()
				end
				
				self.frameCreated = true
			end, 5)
		else
			myButtonFrame = _G.CreateFrame("Frame", "JugheadDistanceAttachmentFrame", _G.UIParent, "JugheadDistanceAttachment")
			myButtonFrame:Show()
			self.frameCreated = true
		end
		
	end
	

	-- If I found GM2, and the user wants me to use it...
	if self.enabled and self.gm2Found then

		DebugPrint( ADDON_NAME .. " " .. Jughead.version .. " " .. Archy.version )

	end

	if self.enabled then
	
		Jughead:RegisterChatCommand("jughead", "MySlashProcessorFunc")

		self:RegisterEvent("PLAYER_ENTERING_WORLD")
		self:RegisterEvent("CURRENCY_DISPLAY_UPDATE")
		self:RegisterEvent("ARTIFACT_DIG_SITE_UPDATED")
		self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")


	end
end

----------------------------------------
----------------------------------------
function Jughead:OnDisable()
    -- Called when the addon is disabled
    if self.enabled then
		self:UnregisterEvent("PLAYER_ENTERING_WORLD")
		self:UnregisterEvent("CURRENCY_DISPLAY_UPDATE")
		self:UnregisterEvent("ARTIFACT_DIG_SITE_UPDATED")
		self:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED")
    	
    	if myButtonFrame then
    		myButtonFrame:Hide()
    	end
    end
    
    if timer_handle then
	    self:CancelTimer(timer_handle)
	end
    
end


----------------------------------------
----------------------------------------
dumpMinimapStats = function()

	local kids = fetchArchySurveyPOI()

	for _, cFrame in pairs(kids) do
		cFrame.icon:Hide()
		cFrame:Hide()
	end

	DebugPrint( "Found " .. #kids .. " Archy survey frames.")
	
end


----------------------------------------
-- Slash command handler
----------------------------------------
function Jughead:MySlashProcessorFunc(input)
  -- Process the slash command ('input' contains whatever follows the slash command)
    if self.enabled then
		local command = input:lower()

		if command == L["dump"]:lower() then
			dumpMinimapStats()
		elseif command == L["red"]:lower() then
			buttonRed_OnLClick()
		elseif command == L["yellow"]:lower() then
			buttonYellow_OnLClick()
		elseif command == L["green"]:lower() then
			buttonGreen_OnLClick()
		elseif command == "isa" then
			userIsOnArchyNode()
		end
    
    end
end



